@@ -26,7 +26,9 @@ import locale  # for multilanguage support
 import gettext
 import gc
 import subprocess
+import ctypes
 from urllib.request import url2pathname
+from functools import lru_cache
 
 
 sharedir = os.path.join(sys.prefix, 'share')
@@ -60,15 +62,14 @@ if hasattr(locale, 'bindtextdomain'):
     except AttributeError:
         pass
 else:
-    from ctypes import cdll
     # Windows or musl
-    libintl = cdll['libintl-8' if os.name == 'nt' else 'libintl.so.8']
+    libintl = ctypes.cdll['libintl-8' if os.name == 'nt' else 'libintl.so.8']
     libintl.bindtextdomain(DOMAIN.encode(), localedir.encode(sys.getfilesystemencoding()))
     libintl.bind_textdomain_codeset(DOMAIN.encode(), 'UTF-8'.encode())
     del libintl
 
 APPNAME = 'PDF Arranger'
-VERSION = '1.7.0'
+VERSION = '1.7.1'
 WEBSITE = 'https://github.com/pdfarranger/pdfarranger'
 
 # Add support for dnd to other instance and insert file at drop location in Windows
@@ -163,6 +164,27 @@ def warn_dialog(func):
     return wrapper
 
 
+def malloc_trim():
+    """Release free memory from the heap."""
+    if os.name == 'nt':
+        return
+    mtrim = malloc_trim_available()
+    if mtrim:
+        mtrim()
+
+
+@lru_cache()
+def malloc_trim_available():
+    try:
+        ctypes.CDLL('libc.so.6').malloc_trim(0)
+    except (FileNotFoundError, AttributeError, OSError):
+        print('malloc_trim not available. Application may not release memory properly.')
+        return
+    def mtrim():
+        ctypes.CDLL('libc.so.6').malloc_trim(0)
+    return mtrim
+
+
 def get_file_path_from_uri(uri):
     """Extracts the path from an uri"""
     uri = uri[5:]  # remove 'file:'
@@ -533,7 +555,7 @@ class PdfArranger(Gtk.Application):
 
         all_files = self.active_file_names()
         if len(all_files) > 0:
-            title += ' [' + ', '.join(all_files) + ']'
+            title += ' [' + ', '.join(sorted(all_files)) + ']'
 
         title += ' – ' + APPNAME
         self.window.set_title(title)
@@ -559,6 +581,8 @@ class PdfArranger(Gtk.Application):
             page.zoom = self.zoom_scale
             page.thumbnail = thumbnail
             self.model[num][0] = page
+        elif num == -2:
+            malloc_trim()
         self.update_progress_bar(num)
 
     def on_window_size_request(self, _window):
@@ -825,6 +849,7 @@ class PdfArranger(Gtk.Application):
         self.iconview.grab_focus()
         if self.progress_bar.get_visible() and len(self.model) > 0:
             self.render()
+        malloc_trim()
 
     def copy_pages(self):
         """Collect data from selected pages"""
@@ -981,6 +1006,8 @@ class PdfArranger(Gtk.Application):
                 self.paste_files(data, before, ref_to)
             else:
                 self.paste_pages(data, before, ref_to, select_added=False)
+            if pastemode == 'BEFORE':
+                self.__update_statusbar()
         elif pastemode in ['ODD', 'EVEN']:
             if data_is_filepaths:
                 # Generate data to send to paste_pages_interleave
@@ -1179,11 +1206,12 @@ class PdfArranger(Gtk.Application):
                              for p in data]
             iter_to = self.model.get_iter(ref_to.get_path())
             for ref_from in ref_from_list:
-                row = model[model.get_iter(ref_from.get_path())]
+                iterator = model.get_iter(ref_from.get_path())
+                page = model.get_value(iterator, 0).duplicate()
                 if before:
-                    it = model.insert_before(iter_to, row[:])
+                    it = model.insert_before(iter_to, [page, page.description()])
                 else:
-                    it = model.insert_after(iter_to, row[:])
+                    it = model.insert_after(iter_to, [page, page.description()])
                 path = model.get_path(it)
                 iconview.select_path(path)
             if move:
@@ -1211,6 +1239,7 @@ class PdfArranger(Gtk.Application):
         for ref_del in ref_del_list:
             path = ref_del.get_path()
             model.remove(model.get_iter(path))
+        malloc_trim()
 
     def iv_dnd_motion(self, iconview, context, x, y, etime):
         """Handles drag motion: autoscroll, select move or copy, select drag cursor location."""
@@ -1545,7 +1574,8 @@ class PdfArranger(Gtk.Application):
 
     def zoom_set(self, level):
         """Sets the zoom level"""
-        if level < -10 or level > 40:
+        level = min(max(level, -10), 40)
+        if level == self.zoom_level:
             return
         if self.zoom_change_render:
             GObject.source_remove(self.zoom_change_render)
@@ -1582,7 +1612,6 @@ class PdfArranger(Gtk.Application):
         selection = self.iconview.get_selected_items()
         if len(selection) != 1:
             return
-        self.zoom_full_page = True
 
         item_padding = self.iconview.get_item_padding()
         cell_image_renderer, cell_text_renderer = self.iconview.get_cells()
@@ -1601,6 +1630,8 @@ class PdfArranger(Gtk.Application):
         zoom_scaleX_new = (sw_width - cell_extraX - margins) / page_width
         zoom_scaleY_new = (sw_height - cell_extraY - margins) / page_height
         self.zoom_scale = min(zoom_scaleY_new, zoom_scaleX_new)
+        if self.zoom_scale < 0.2 * (1.1 ** -10):
+            return
         for page, _ in self.model:
             page.zoom = self.zoom_scale
         selected_page_nr = Gtk.TreePath.get_indices(selection[0])[0]
@@ -1610,6 +1641,7 @@ class PdfArranger(Gtk.Application):
         self.zoom_level = -10
         while self.zoom_scale > 0.2 * (1.1 ** self.zoom_level):
             self.zoom_level += 1
+        self.zoom_full_page = True
 
     def scroll_to_selection(self):
         """Scroll iconview so that selection is in center of window."""
